@@ -7,39 +7,50 @@
 
 #include <KActionMenu>
 #include <kcolorschememanager.h>
+#include <kcolorschememenu.h>
 
 #include <QApplication>
 #include <QDialog>
 #include <QDialogButtonBox>
 #include <QListView>
+#include <QMainWindow>
 #include <QMenu>
+#include <QMenuBar>
 #include <QToolButton>
 #include <QVBoxLayout>
 
-class KColorSchemeDemo : public QDialog
+class KColorSchemeDemo : public QMainWindow
 {
     Q_OBJECT
 public:
     KColorSchemeDemo()
-        : QDialog(nullptr)
+        : QMainWindow(nullptr)
     {
         KColorSchemeManager *manager = new KColorSchemeManager(this);
 
         QListView *view = new QListView(this);
         view->setModel(manager->model());
         connect(view, &QListView::activated, manager, &KColorSchemeManager::activateScheme);
+        manager->setAutosaveChanges(true);
 
         QDialogButtonBox *box = new QDialogButtonBox(QDialogButtonBox::Close, this);
         connect(box, &QDialogButtonBox::rejected, qApp, &QApplication::quit);
 
         QToolButton *button = new QToolButton(box);
         button->setIcon(QIcon::fromTheme(QStringLiteral("fill-color")));
-        button->setMenu(manager->createSchemeSelectionMenu(QStringLiteral("Oxygen"), button)->menu());
+        button->setMenu(KColorSchemeMenu::createMenu(manager, button)->menu());
         box->addButton(button, QDialogButtonBox::InvalidRole);
 
-        QVBoxLayout *layout = new QVBoxLayout(this);
+        QWidget *w = new QWidget();
+        QVBoxLayout *layout = new QVBoxLayout(w);
         layout->addWidget(view);
         layout->addWidget(box);
+
+        setCentralWidget(w);
+
+        QMenu *menu = new QMenu("Menu", this);
+        menu->addAction(KColorSchemeMenu::createMenu(manager, this));
+        menuBar()->addMenu(menu);
     }
     ~KColorSchemeDemo() override
     {
